# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-18 15:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('visaweb', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Command',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=200)),
                ('name', models.CharField(max_length=200)),
                ('subject', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='actions', to='visaweb.Asset')),
            ],
        ),
        migrations.AddField(
            model_name='property',
            name='getter_command',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='getter_command', to='visaweb.Command'),
        ),
        migrations.AddField(
            model_name='property',
            name='setter_command',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='setter_command', to='visaweb.Command'),
        ),
    ]



from django.contrib import admin

# Register your models here.
from django import forms

from .models import Asset
from .models import Command
from .models import Property
from .models import ChoiceProperty
from .models import IntervalProperty
from .models import StringProperty
from .models import EnumerationType
from .models import EnumerationChoice



class EnumerationChoiceField(forms.ModelChoiceField):
     def label_from_instance(self, obj):
         return "%s (%s)" % (obj.display_name, obj.id)

class ChoicePropertyAdminForm(forms.ModelForm):
    # fields

    class Meta:
        model = ChoiceProperty
        fields = "__all__"

    def __init__(self, *args, **kwargs):
        super(ChoicePropertyAdminForm, self).__init__(*args, **kwargs)
        current_default_unit = self.instance.default_unit
        self.fields['default_unit'].choices = EnumerationChoiceField(queryset=EnumerationChoice.objects.all())

class IntervalPropertyInline(admin.TabularInline):
    model = IntervalProperty
    fk_name = 'owner'
    extra = 1


class ChoicePropertyInline(admin.TabularInline):
    model = ChoiceProperty
    fk_name = 'owner'
    # form = ChoicePropertyAdminForm
    extra = 1

class StringPropertyInline(admin.TabularInline):
    model = StringProperty
    fk_name = 'owner'
    # form = ChoicePropertyAdminForm
    extra = 1

class CommandInline(admin.StackedInline):
    model = Command
    fk_name = 'subject'
    # form = ChoicePropertyAdminForm
    extra = 1

class CommandAdmin(admin.ModelAdmin):
    list_display = ('id', 'code', 'name')


class AssetAdmin(admin.ModelAdmin):
    # ...
    # form = EnumerationChoiceAdminForm

    list_display = ('id', 'display_name', 'alias', 'address', 'description')
    inlines = [IntervalPropertyInline, ChoicePropertyInline, StringPropertyInline, CommandInline]


class EnumerationChoiceInline(admin.TabularInline):
    model = EnumerationChoice
    extra = 1

class EnumerationTypeAdmin(admin.ModelAdmin):
    # ...
    inlines = [EnumerationChoiceInline]
    list_display = ('id', 'display_name', 'description')

# admin.site.register(EnumerationChoice)
admin.site.register(EnumerationType, EnumerationTypeAdmin)

admin.site.register(Asset, AssetAdmin)
admin.site.register(Command, CommandAdmin)
admin.site.register(ChoiceProperty)
admin.site.register(IntervalProperty)


import logging
import sys
import yaml

def setupLogging(logging_level, logfile):
    root = logging.getLogger()
    root.setLevel(logging_level)

    logging.basicConfig(filename=logfile, level=logging_level)

    # ch = logging.StreamHandler(sys.stdout)
    # ch.setLevel(logging_level)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # root.setFormatter(formatter)
    # root.addHandler(ch)
    return root;

def loadConfiguration(externalizedFilename, defaults):
    try:
        with open(externalizedFilename, 'r') as ymlfile:
            configuration = yaml.load(ymlfile)

        logging.info('Configuration loaded.')
    except IOError as e:
        logging.warning(e)
        try:
            logging.info('Initialize new configuration ...')
            with open(externalizedFilename, 'w') as ymlfile:
                configuration = yaml.dump(defaults, ymlfile, default_flow_style=False)
            logging.info('Default configuration created.')
        except IOError:
            raise EnvironmentError('Configuration file mq could not created.')
    return configuration


import argparse
import logging

from rpc.server import VisaRpcServer

def main():
    ## Parse command line arguments.
    parser = argparse.ArgumentParser()

    parser.add_argument("-H", "--host", help="Automation server host of field agent.", type=str, default='localhost')
    parser.add_argument("-V", "--backend", help="PyVISA backend.", type=str, default='test.yml@sim')
    parser.add_argument("-e", "--exchange", help="RabbitMQ exchange for RPC.", type=str, default='visa')
    parser.add_argument("-u", "--username", help="RabbitMQ username.", type=str, default='guest')
    parser.add_argument("-P", "--port", help="RabbitMQ port.", type=int, default=5672)
    parser.add_argument("-p", "--password", help="RabbitMQ password.", type=str, default='guest')
    parser.add_argument("-v", "--verbose", help="Increase output verbosity.", action="store_true")
    parser.add_argument("-l", "--logfile", help="Logging to file.", type=str, default="/opt/hedgehog-station-controller/output.log")

    args = parser.parse_args()

    rootlogger = logging.getLogger('root')
    # define a Handler which writes INFO messages or higher to the sys.stderr
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    # set a format which is simpler for console use
    formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
    # tell the handler to use this fo
    console.setFormatter(formatter)
    # add the handler to the root logger
    rootlogger.addHandler(console)

    logger = logging.getLogger('assetadapter.main')
    logfile = logging.FileHandler(args.logfile)
    logfile.setLevel(logging.DEBUG)
    # set a format which is simpler for console use
    logfile_formatter = logging.Formatter('%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
    # tell the handler to use this fo
    logfile.setFormatter(logfile_formatter)
    # add the handler to the root logger
    logger.addHandler(logfile)

    logger.setLevel(logging.DEBUG)

    logger.info("Initialize controller service with configuration: " + str(args))

    # Initialize VISA RPC server
    server = VisaRpcServer(
        logger=logger,
        host=args.host,
        username=args.username,
        password=args.password,
        visa_library=args.backend,
        exchange=args.exchange
    )

    server.start()


